// *=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*
// ** Copyright UCAR (c) 1992 - 2007
// ** University Corporation for Atmospheric Research (UCAR)
// ** National Center for Atmospheric Research (NCAR)
// ** Research Applications Lab (RAL)
// ** P.O.Box 3000, Boulder, Colorado, 80307-3000, USA
// *=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*

 

////////////////////////////////////////////////////////////////////////


#ifndef  __MET_ANALYSIS_MODE_LINE_H__
#define  __MET_ANALYSIS_MODE_LINE_H__


////////////////////////////////////////////////////////////////////////


#include <iostream>
#include <fstream>

#include "vx_analysis_util/data_line.h"

#include "vx_util/vx_util.h"
#include "vx_cal/vx_cal.h"


////////////////////////////////////////////////////////////////////////


class ModeLine : public DataLine {

   public:

      ModeLine();
     ~ModeLine();
      ModeLine(const ModeLine &);
      ModeLine & operator=(const ModeLine &);


      // void clear();

      void dump(ostream &, int depth = 0) const;

      int read_line(LineDataFile *);   //  virtual from base class

      int is_ok() const;   //  virtual from base class

         //
         //  retrieve stuff
         //

      const char * model                      () const;   //  column  1

      int          fcst_lead                  () const;   //  column  2
      unixtime     fcst_valid                 () const;   //  column  3
      int          fcst_accum                 () const;   //  column  4

      int          lead2                      () const;   //  column  5
      unixtime     obs_valid                  () const;   //  column  6
      int          obs_accum                  () const;   //  column  7

      int          fcst_rad                   () const;   //  column  8
      const char * fcst_thr                   () const;   //  column  9
      int          obs_rad                    () const;   //  column 10
      const char * obs_thr                    () const;   //  column 11

      const char * fcst_var                   () const;   //  column 12
      const char * fcst_lev                   () const;   //  column 13
      const char * obs_var                    () const;   //  column 14
      const char * obs_lev                    () const;   //  column 15

      const char * object_id                  () const;   //  column 16
      const char * object_cat                 () const;   //  column 17

         //  skip the "=" sign in column 18

      double       centroid_x                 () const;   //  column 19
      double       centroid_y                 () const;   //  column 20
      double       centroid_lat               () const;   //  column 21
      double       centroid_lon               () const;   //  column 22

      double       axis_ang                   () const;   //  column 23

      double       length                     () const;   //  column 24
      double       width                      () const;   //  column 25

      int          area                       () const;   //  column 26
      int          area_filter                () const;   //  column 27

      double       curvature                  () const;   //  column 28
      double       curvature_x                () const;   //  column 29
      double       curvature_y                () const;   //  column 30

      double       complexity                 () const;   //  column 31

      double       intensity_10               () const;   //  column 32
      double       intensity_25               () const;   //  column 33
      double       intensity_50               () const;   //  column 34
      double       intensity_75               () const;   //  column 35
      double       intensity_90               () const;   //  column 36
      double       intensity_user             () const;   //  column 37

      double       intensity_sum              () const;   //  column 38
      double       centroid_dist              () const;   //  column 39
      double       boundary_dist              () const;   //  column 40
      double       convex_hull_dist           () const;   //  column 41
      double       angle_diff                 () const;   //  column 42

      double       area_ratio                 () const;   //  column 43
      int          intersection_area          () const;   //  column 44
      int          union_area                 () const;   //  column 45
      int          symmetric_diff             () const;   //  column 46
      double       intersection_over_area     () const;   //  column 47

      double       complexity_ratio           () const;   //  column 48
      double       percentile_intensity_ratio () const;   //  column 49

      double       interest                   () const;   //  column 50

         //
         //
         //

      int          is_fcst                    () const;
      int          is_obs                     () const;

      int          is_single                  () const;
      int          is_pair                    () const;

      int          is_simple                  () const;
      int          is_composite               () const;

      int          is_matched                 () const;
      int          is_unmatched               () const;


};


////////////////////////////////////////////////////////////////////////


extern StringArray get_mode_filenames(const StringArray &);

extern StringArray get_mode_filenames_from_dir(const char * directory_path);

extern int is_mode_filename(const char * path);


////////////////////////////////////////////////////////////////////////


#endif   /*  __MET_ANALYSIS_MODE_LINE_H__  */


////////////////////////////////////////////////////////////////////////


