// *=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*
// ** Copyright UCAR (c) 1992 - 2007
// ** University Corporation for Atmospheric Research (UCAR)
// ** National Center for Atmospheric Research (NCAR)
// ** Research Applications Lab (RAL)
// ** P.O.Box 3000, Boulder, Colorado, 80307-3000, USA
// *=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*



////////////////////////////////////////////////////////////////////////


#ifndef  __MET_ANALYSIS_VSDB_H__
#define  __MET_ANALYSIS_VSDB_H__


////////////////////////////////////////////////////////////////////////


#include <iostream>
#include <fstream>

#include "vx_analysis_util/data_line.h"

#include "vx_util/vx_util.h"
#include "vx_cal/vx_cal.h"


////////////////////////////////////////////////////////////////////////


   //
   //  Enumerate all the possible vsdb line types
   //


enum VSDBLineType {

      //
      //  NCEP VSDB Line Types
      //

   vsdb_sl1l2  = 0,
   vsdb_sal1l2 = 1,
   vsdb_vl1l2  = 2,
   vsdb_val1l2 = 3,

   vsdb_fho    = 4,

      //
      //  MET VSDB Line Types
      //

   vsdb_ctc    = 5,
   vsdb_ctp    = 6,
   vsdb_cfp    = 7,
   vsdb_cop    = 8,

   vsdb_cts    = 9,
   vsdb_cnt    = 10,
   vsdb_mpr    = 11,

   vsdb_nbrctc = 12,
   vsdb_nbrcts = 13,
   vsdb_nbrcnt = 14,

   vsdb_isc    = 15,

      //
      //  flag value
      //

   no_vsdb_line_type = 16

};


////////////////////////////////////////////////////////////////////////


class VSDBLine : public DataLine {

   private:

      void determine_line_type();

      VSDBLineType Type;

   public:

      VSDBLine();
     ~VSDBLine();
      VSDBLine(const VSDBLine &);
      VSDBLine & operator=(const VSDBLine &);


      // void clear();

      void dump(ostream &, int depth = 0) const;

         //
         //  retrieve stuff
         //

      VSDBLineType type  () const;

      unixtime init_time () const;   //  valid time - fcst hour

      int      init_hour () const;   //  init_time % sec_per_day


         //
         //  retrieve values of the first 9 columns
         //  and last 2 columns for the interpolation method
         //


      int          version    () const;

      const char * model      () const;

      int          fcst_lead  () const;

      unixtime     valid_time () const;

      const char * obtype     () const;

      const char * vx_mask    () const;

      const char * line_type  () const;

      const char * var        () const;

      const char * level      () const;

      const char * interp_mthd() const;

      int          interp_pnts() const;

         //
         //
         //

      virtual int read_line(LineDataFile *);

};


////////////////////////////////////////////////////////////////////////


inline  VSDBLineType  VSDBLine::type () const { return ( Type ); }


////////////////////////////////////////////////////////////////////////


extern StringArray get_vsdb_filenames(const StringArray &);

extern StringArray get_vsdb_filenames_from_dir(const char * directory_path);

extern int is_vsdb_filename(const char * path);

extern VSDBLineType determine_line_type(const char *);

extern int determine_column_offset(VSDBLineType, const char *);


////////////////////////////////////////////////////////////////////////


#endif   /*  __MET_ANALYSIS_VSDB_H__  */


////////////////////////////////////////////////////////////////////////


