// *=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*
// ** Copyright UCAR (c) 1992 - 2007
// ** University Corporation for Atmospheric Research (UCAR)
// ** National Center for Atmospheric Research (NCAR)
// ** Research Applications Lab (RAL)
// ** P.O.Box 3000, Boulder, Colorado, 80307-3000, USA
// *=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*

////////////////////////////////////////////////////////////////////////

#ifndef  __VX_COMPUTE_CI_H__
#define  __VX_COMPUTE_CI_H__

////////////////////////////////////////////////////////////////////////

#include "vx_gsl_prob/vx_gsl_prob.h"
#include "vx_util/vx_util.h"
#include "vx_met_util/met_stats.h"

////////////////////////////////////////////////////////////////////////

static const int large_sample_threshold = 30;
static const int wald_sample_threshold  = 100;

////////////////////////////////////////////////////////////////////////

extern void compute_proportion_ci(double p, int n, double alpha, double &p_cl, double &p_cu);

extern void compute_wald_ci(double p, int n, double alpha, double &p_cl, double &p_cu);

extern void compute_wilson_ci(double p, int n, double alpha, double &p_cl, double &p_cu);

extern void compute_woolf_ci(double odds, double alpha, 
                             int fy_oy, int fy_on, int fn_oy, int fn_on,
                             double &odds_cl, double &odds_cu);

extern void compute_hk_ci(double hk, double alpha,
                          int fy_oy, int fy_on, int fn_oy, int fn_on,
                          double &hk_cl, double &hk_cu);

extern void compute_stats_ci_bca(const gsl_rng *,
                                 const NumArray &, const NumArray &,
                                 int, int,
                                 CNTInfo &, CTSInfo *&, int, int,
                                 const char *);

extern void compute_stats_ci_perc(const gsl_rng *,
                                  const NumArray &, const NumArray &,
                                  int, int, double,
                                  CNTInfo &, CTSInfo *&, int, int,
                                  const char *);

extern void compute_nbr_stats_ci_bca(const gsl_rng *,
                                     const NumArray &, const NumArray &,
                                     int, int,
                                     NBRCNTInfo &, NBRCTSInfo *&, int,
                                     const char *);

extern void compute_nbr_stats_ci_perc(const gsl_rng *,
                                      const NumArray &, const NumArray &,
                                      int, int, double,
                                      NBRCNTInfo &, NBRCTSInfo *&, int,
                                      const char *);

extern void compute_mean_stdev_ci_bca(const gsl_rng *, const NumArray &,
                                      int, double,
                                      CIInfo &, CIInfo &);

extern void compute_mean_stdev_ci_perc(const gsl_rng *, const NumArray &,
                                       int, double, double,
                                       CIInfo &, CIInfo &);

extern void compute_bca_interval(double, NumArray &, NumArray &,
                                 double, double &, double &);

extern void compute_perc_interval(double, NumArray &,
                                  double, double &, double &);

////////////////////////////////////////////////////////////////////////

#endif   // __VX_COMPUTE_CI_H__

////////////////////////////////////////////////////////////////////////
