// *=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*
// ** Copyright UCAR (c) 1992 - 2007
// ** University Corporation for Atmospheric Research (UCAR)
// ** National Center for Atmospheric Research (NCAR)
// ** Research Applications Lab (RAL)
// ** P.O.Box 3000, Boulder, Colorado, 80307-3000, USA
// *=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*


////////////////////////////////////////////////////////////////////////

using namespace std;

#include <iostream>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <cmath>

#include "vx_met_util/rescale_lon.h"
#include "vx_math/constants.h"

////////////////////////////////////////////////////////////////////////
//
// Rescale the longitude value provided to be in the range from
// -180 to 180
//
////////////////////////////////////////////////////////////////////////

double rescale_lon(double lon) {
   double new_lon;

   // Check for bad data
   if(lon == bad_data_double)
      return(bad_data_double);

   new_lon = lon;
   if(new_lon < -180.0) {
      while(new_lon < -180.0) new_lon += 360.0;
   }
   if(new_lon > 180.0) {
      while(new_lon >  180.0) new_lon -= 360.0;
   }

   return(new_lon);
}

////////////////////////////////////////////////////////////////////////
//
// Rescale the degrees provided to be in the range specified
//
////////////////////////////////////////////////////////////////////////

double rescale_deg(double deg, double lower, double upper) {
   double new_deg;

   // Check for bad data
   if(deg == bad_data_double)
      return(bad_data_double);

   if(abs(upper - lower) != 360.0 || lower > upper) {
      cerr << "\n\nERROR: rescale_deg() -> "
           << "invalid upper and lower limits supplied.\n\n"
           << flush;
      exit(1);
   }

   new_deg = deg;
   if(new_deg < lower) {
      while(new_deg < lower) new_deg += 360.0;
   }
   if(new_deg > upper) {
      while(new_deg > upper) new_deg -= 360.0;
   }

   return(new_deg);
}

////////////////////////////////////////////////////////////////////////
//
// Compute the number of degrees between two angles
//
////////////////////////////////////////////////////////////////////////

double angle_between(double a, double b) {
   double d, d1, d2;

   // Check for bad data
   if(a == bad_data_double || b == bad_data_double)
      return(bad_data_double);

   // Compute the difference between the angles, and rescale them from
   // 0 to 360
   d1 = rescale_deg(a - b, 0.0, 360.0);
   d2 = rescale_deg(b - a, 0.0, 360.0);

   if(min(d1, d2) < 90.0) d = min(d1, d2);
   else                   d = max(d1, d2) - 180.0;

   return(d);
}

////////////////////////////////////////////////////////////////////////
