// *=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*
// ** Copyright UCAR (c) 1992 - 2007
// ** University Corporation for Atmospheric Research (UCAR)
// ** National Center for Atmospheric Research (NCAR)
// ** Research Applications Lab (RAL)
// ** P.O.Box 3000, Boulder, Colorado, 80307-3000, USA
// *=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*



////////////////////////////////////////////////////////////////////////


using namespace std;

#include <iostream>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <cmath>

#include "vx_util/ascii_table.h"


////////////////////////////////////////////////////////////////////////


static const int ics_max = 100;   //  maximum value of intercolumn space
                                  //  should be plenty big enough

static const int irs_max = 100;   //  maximum value of interrow space
                                  //  should be plenty big enough

////////////////////////////////////////////////////////////////////////


static void  right_justified_item(ostream &, const char *, int width, char pad);
static void   left_justified_item(ostream &, const char *, int width, char pad);
static void center_justified_item(ostream &, const char *, int width, char pad);


////////////////////////////////////////////////////////////////////////


   //
   //  Code for class AsciiTable
   //


////////////////////////////////////////////////////////////////////////


AsciiTable::AsciiTable()

{

init_from_scratch();

}


////////////////////////////////////////////////////////////////////////


AsciiTable::~AsciiTable()

{

clear();

}


////////////////////////////////////////////////////////////////////////


AsciiTable::AsciiTable(const AsciiTable & a)

{

init_from_scratch();

assign(a);

}


////////////////////////////////////////////////////////////////////////


AsciiTable & AsciiTable::operator=(const AsciiTable & a)

{

if ( this == &a )  return ( * this );

assign(a);

return ( * this );

}


////////////////////////////////////////////////////////////////////////


void AsciiTable::init_from_scratch()

{

e = (char **) 0;

ColWidth = (int *) 0;

Just = (AsciiTableColJust *) 0;

clear();

return;

}


////////////////////////////////////////////////////////////////////////


void AsciiTable::clear()

{

if ( e )  {

   int j;

   for (j=0; j<(Nrows*Ncols); ++j)  {

      if ( e[j] )  { delete [] e[j];  e[j] = (char *) 0; }

   }

   delete [] e;  e = (char **) 0;

}

if ( ColWidth )  { delete [] ColWidth;  ColWidth = (int *) 0; }

if ( Just )  { delete [] Just;  Just = (AsciiTableColJust *) 0; }

InterColumnSpace = 1;   //  not zero
InterRowSpace    = 0;

ColSepChar = ' ';

Nrows = Ncols = 0;

return;

}


////////////////////////////////////////////////////////////////////////


int AsciiTable::rc_to_n(int r, int c) const

{

if ( !e )  {

   cerr << "\n\n  AsciiTable::rc_to_n() -> empty table!\n\n";

   exit ( 1 );

}

if ( (r < 0) || (r >= Nrows) || (c < 0) || (c >= Ncols) )  {

   cerr << "\n\n  AsciiTable::rc_to_n() -> range check error!\n\n";

   exit ( 1 );

}

int n;

n = r*Ncols + c;

return ( n );

}


////////////////////////////////////////////////////////////////////////


void AsciiTable::assign(const AsciiTable & a)

{

clear();

if ( !(a.e) )  return;

set_size_rc(a.nrows(), a.ncols());

set_ics(a.intercolumn_space());
set_irs(a.interrow_space());

int r, c;

for (r=0; r<Nrows; ++r)  {

   for (c=0; c<Ncols; ++c)  {

      set_entry(r, c, a(r, c));

   }

}

for (c=0; c<Ncols; ++c)  ColWidth[c] = a.ColWidth[c];

for (c=0; c<Ncols; ++c)  Just[c] = a.Just[c];

ColSepChar = a.ColSepChar;

   //
   //  done
   //

return;

}


////////////////////////////////////////////////////////////////////////


void AsciiTable::set_size_rc(const int NR, const int NC)

{

clear();

int j;

e = new char * [NR*NC];

if ( !e )  {

   cerr << "\n\n  AsciiTable::set_size_rc() -> memory allocation error 1\n\n";

   exit ( 1 );

}

for (j=0; j<(NR*NC); ++j)  e[j] = (char *) 0;

Nrows = NR;
Ncols = NC;

ColWidth = new int [Ncols];

if ( !ColWidth )  {

   cerr << "\n\n  AsciiTable::set_size_rc() -> memory allocation error 2\n\n";

   exit ( 1 );

}

for (j=0; j<Ncols; ++j)  ColWidth[j] = 0;

Just = new AsciiTableColJust [Ncols];

if ( !Just )  {

   cerr << "\n\n  AsciiTable::set_size_rc() -> memory allocation error 3\n\n";

   exit ( 1 );

}

for (j=0; j<Ncols; ++j)  Just[j] = at_ColRight;

   //
   //  done
   //

return;

}


////////////////////////////////////////////////////////////////////////


void AsciiTable::set_ics(int k)

{

if ( (k < 0) || (k > ics_max) )  {

   cerr << "\n\n   AsciiTable::set_ics(int) -> bad value ... " << k << "\n\n";

   exit ( 1 );

}

InterColumnSpace = k;

return;

}


////////////////////////////////////////////////////////////////////////


void AsciiTable::set_irs(int k)

{

if ( (k < 0) || (k > irs_max) )  {

   cerr << "\n\n   AsciiTable::set_irs(int) -> bad value ... " << k << "\n\n";

   exit ( 1 );

}

InterRowSpace = k;

return;

}


////////////////////////////////////////////////////////////////////////


void AsciiTable::set_col_sep_char(const char c)

{

ColSepChar = c;

return;

}


////////////////////////////////////////////////////////////////////////


void AsciiTable::set_table_just(const AsciiTableColJust actj)

{

int j;

for (j=0; j<Ncols; ++j)  Just[j] = actj;

return;

}


////////////////////////////////////////////////////////////////////////


void AsciiTable::set_col_just(int c, const AsciiTableColJust actj)

{

if ( (c < 0) || (c >= Ncols) )  {

   cerr << "\n\n  AsciiTable::set_col_just() -> range check error!\n\n";

   exit ( 1 );

}


Just[c] = actj;

return;

}


////////////////////////////////////////////////////////////////////////


AsciiTableColJust AsciiTable::col_just(int c) const

{

if ( (c < 0) || (c >= Ncols) )  {

   cerr << "\n\n  AsciiTable::col_just(int) -> range check error!\n\n";

   exit ( 1 );

}


return ( Just[c] );

}


////////////////////////////////////////////////////////////////////////


void AsciiTable::set_entry(const int r, const int c, const char * text)

{


int n, k;

n = rc_to_n(r, c);   //  rc_to_n does range checking on r and c, 
                     //    so we don't need to that here

   //
   //  delete old entry, if any
   //

if ( e[n] )  { delete [] e[n];  e[n] = (char *) 0; }

k = strlen(text);

e[n] = new char [1 + k];

if ( !(e[n]) )  {

   cerr << "\n\n  AsciiTable::set_entry() -> memory allocation error\n\n";

   exit ( 1 );

}

memset(e[n], 0, 1 + k);

strcpy(e[n], text);

if ( ColWidth[c] < k )  ColWidth[c] = k;

   //
   //  done
   //

return;

}


////////////////////////////////////////////////////////////////////////


const char * AsciiTable::operator()(int r, int c) const

{

int n;

n = rc_to_n(r, c);   //  rc_to_n does range checking on r and c, 
                     //    so we don't need to that here

return ( e[n] );

}

////////////////////////////////////////////////////////////////////////


int AsciiTable::line_length() const

{

if ( !e )  return ( 0 );

int j, L;

L = 0;

for (j=0; j<Ncols; ++j)  L += ColWidth[j];

L += (Ncols - 1)*InterColumnSpace;


return ( L );

}


////////////////////////////////////////////////////////////////////////


int AsciiTable::col_width(int k) const

{

if ( (k < 0) || (k >= Ncols) )  {

   cerr << "\n\n  AsciiTable::col_width(int) const -> range check error\n\n";

   exit ( 1 );

}

return ( ColWidth[k] );

}


////////////////////////////////////////////////////////////////////////


   //
   //  Code for misc functions
   //


////////////////////////////////////////////////////////////////////////


ostream & operator<<(ostream & out, const AsciiTable & t)

{

int j;
int r, c;
const char * text = (const char *) 0;
const char pad = ' ';


for (r=0; r<(t.nrows()); ++r)  {

   if ( r > 0 )  {

      for (j=0; j<(t.interrow_space()); ++j)  out.put('\n');

   }

   for (c=0; c<(t.ncols()); ++c)  {

      if ( c > 0 )  {

         for (j=0; j<(t.intercolumn_space()); ++j)  out.put(t.col_sep_char());

      }

      text = t(r, c);

      switch ( t.col_just(c) )  {

         case at_ColRight:
            right_justified_item(out, text, t.col_width(c), pad);
            break;

         case at_ColLeft:
            left_justified_item(out, text, t.col_width(c), pad);
            break;

         case at_ColCenter:
            center_justified_item(out, text, t.col_width(c), pad);
            break;

         default:
            cerr << "\n\n  operator<<(ostream &, const AsciiTable &) -> bad column justification\n\n";
            exit ( 1 );
            break;

      }   //  switch

   }   //  for c

   out.put('\n');

}   //  for r

   //
   //  done
   //

out.flush();

return ( out );

}


////////////////////////////////////////////////////////////////////////


void right_justified_item(ostream & out, const char * text, int width, char pad)

{

int j;
int len;


len = strlen(text);

for (j=len; j<width; ++j)  out.put(pad);

out << text;


   //
   //  done
   //

return;

}


////////////////////////////////////////////////////////////////////////


void left_justified_item(ostream & out, const char * text, int width, char pad)

{

int j;
int len;


len = strlen(text);

out << text;

for (j=len; j<width; ++j)  out.put(pad);



   //
   //  done
   //

return;

}


////////////////////////////////////////////////////////////////////////


void center_justified_item(ostream & out, const char * text, int width, char pad)

{

int w2;
int j;
int len;


len = strlen(text);

w2 = (width - len)/2;

for (j=0; j<w2; ++j)  out.put(pad);

out << text;

for (j=(len + w2); j<width; ++j)  out.put(pad);



   //
   //  done
   //

return;

}


////////////////////////////////////////////////////////////////////////








