// *=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*
// ** Copyright UCAR (c) 1992 - 2007
// ** University Corporation for Atmospheric Research (UCAR)
// ** National Center for Atmospheric Research (NCAR)
// ** Research Applications Lab (RAL)
// ** P.O.Box 3000, Boulder, Colorado, 80307-3000, USA
// *=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*



////////////////////////////////////////////////////////////////////////


#ifndef  __VX_ASCII_TABLE_H__
#define  __VX_ASCII_TABLE_H__


////////////////////////////////////////////////////////////////////////


#include <iostream>


////////////////////////////////////////////////////////////////////////


enum AsciiTableColJust {

   at_ColRight,    //  right-justified column
   at_ColLeft,     //  left-justified column
   at_ColCenter,   //  center-justified column

};


////////////////////////////////////////////////////////////////////////


class AsciiTable {

   private:

      void init_from_scratch();

      void assign(const AsciiTable &);

      int rc_to_n(int r, int c) const;

      int Nrows;
      int Ncols;

      int InterColumnSpace;
      int InterRowSpace;

      char ColSepChar;

      char ** e;   //  table entries

      int * ColWidth;

      AsciiTableColJust * Just;

   public:

      AsciiTable();
     ~AsciiTable();
      AsciiTable(const AsciiTable &);
      AsciiTable & operator=(const AsciiTable &);

      void clear();

      void set_size_rc(const int NR, const int NC);

      void set_entry(const int r, const int c, const char * text);

      void set_table_just(const AsciiTableColJust);

      void set_col_just(const int c, const AsciiTableColJust);

      void set_ics(int);
      void set_irs(int);

      void set_col_sep_char(const char);

      int nrows() const;

      int ncols() const;

      const char * operator()(int r, int c) const;

      int intercolumn_space() const;
      int interrow_space() const;

      int col_width(int c) const;

      int line_length() const;

      char col_sep_char() const;

      AsciiTableColJust col_just(int c) const;

};


////////////////////////////////////////////////////////////////////////


inline int AsciiTable::nrows() const { return ( Nrows ); }

inline int AsciiTable::ncols() const { return ( Ncols ); }

inline int AsciiTable::intercolumn_space() const { return ( InterColumnSpace ); }
inline int AsciiTable::interrow_space   () const { return ( InterRowSpace ); }

inline char AsciiTable::col_sep_char() const { return ( ColSepChar ); }


////////////////////////////////////////////////////////////////////////


ostream & operator<<(ostream &, const AsciiTable &);


////////////////////////////////////////////////////////////////////////


#endif   //  __VX_ASCII_TABLE_H__


////////////////////////////////////////////////////////////////////////


