// *=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*
// ** Copyright UCAR (c) 1992 - 2007
// ** University Corporation for Atmospheric Research (UCAR)
// ** National Center for Atmospheric Research (NCAR)
// ** Research Applications Lab (RAL)
// ** P.O.Box 3000, Boulder, Colorado, 80307-3000, USA
// *=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*



////////////////////////////////////////////////////////////////////////


using namespace std;

#include <iostream>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <cmath>

#include "vx_util/concat_string.h"


////////////////////////////////////////////////////////////////////////


   //
   //  Code for class ConcatString
   //


////////////////////////////////////////////////////////////////////////


ConcatString::ConcatString()

{

init_from_scratch();

}


////////////////////////////////////////////////////////////////////////


ConcatString::~ConcatString()

{

clear();

}


////////////////////////////////////////////////////////////////////////


ConcatString::ConcatString(const ConcatString & c)

{

init_from_scratch();

assign(c);

}


////////////////////////////////////////////////////////////////////////


ConcatString::ConcatString(const char * Text)

{

init_from_scratch();

add(Text);

}


////////////////////////////////////////////////////////////////////////


ConcatString & ConcatString::operator=(const ConcatString & c)

{

if ( this == &c )  return ( * this );

assign(c);

return ( * this );

}


////////////////////////////////////////////////////////////////////////


ConcatString & ConcatString::operator=(const char * Text)

{

if ( s == Text )  return ( * this );

clear();

add(Text);

return ( * this );

}


////////////////////////////////////////////////////////////////////////


void ConcatString::init_from_scratch()

{

s = (char *) 0;

clear();

return;

}


////////////////////////////////////////////////////////////////////////


void ConcatString::clear()

{

if ( s )  { delete [] s;  s = (char *) 0; }

Length = Nalloc = 0;

set_precision(concat_string_default_precision);

return;

}


////////////////////////////////////////////////////////////////////////


void ConcatString::assign(const ConcatString & c)

{

clear();

if ( !(c.s) )  return;

extend(1 + c.Length);

memcpy(s, c.s, c.Length);

Length = c.Length;

return;

}


////////////////////////////////////////////////////////////////////////


void ConcatString::extend(int n)

{

if ( n < Nalloc )  return;

int k;

k = n/concat_string_alloc_inc;

if ( n%concat_string_alloc_inc )  ++k;

n = k*concat_string_alloc_inc;

char * u = (char *) 0;

u = new char [n];

if ( !u )  {

   cerr << "\n\n  ConcatString::extend(int) -> memory allocation error\n\n";

   exit ( 1 );

}

memset(u, 0, n);

if ( s && (Length > 0) )  {

   memcpy(u, s, Length);

   delete [] s;  s = (char *) 0;

}

s = u;  u = (char *) 0;

Nalloc = n;

return;

}


////////////////////////////////////////////////////////////////////////


void ConcatString::add(const char * a)

{

int n = strlen(a);


extend(Length + n + 5);   //  just to make sure

strcpy(s + Length, a);

Length += n;


return;

}


////////////////////////////////////////////////////////////////////////


void ConcatString::add(const ConcatString & a)

{

add(a.text());

return;

}


////////////////////////////////////////////////////////////////////////


void ConcatString::chomp()

{

if ( Length == 0 )  return;

if ( s[Length - 1] == '\n' )  {

   --Length;

   s[Length] = (char) 0;

}



return;
}


////////////////////////////////////////////////////////////////////////


void ConcatString::set_precision(int k)

{

if ( (k < 0) || (k > concat_string_max_precision) )  {

   cerr << "\n\n  ConcatString::set_precision(int) -> bad value\n\n";

   exit ( 1 );

}


Precision = k;

memset(FloatFormat, 0, sizeof(FloatFormat));

sprintf(FloatFormat, "%%.%df", Precision);

return;

}


////////////////////////////////////////////////////////////////////////


   //
   //  Code for misc functions
   //


////////////////////////////////////////////////////////////////////////


ConcatString & operator<<(ConcatString & cs, const char c)

{

char s[2];

s[0] = c;
s[1] = (char) 0;

cs.add(s);

return ( cs );

}


////////////////////////////////////////////////////////////////////////


ConcatString & operator<<(ConcatString & cs, const char * s)

{


cs.add(s);


return ( cs );

}


////////////////////////////////////////////////////////////////////////


ConcatString & operator<<(ConcatString & a, const ConcatString & b)

{


a.add(b);


return ( a );

}


////////////////////////////////////////////////////////////////////////


ConcatString & operator<<(ConcatString & a, int k)

{

char junk[64];

sprintf(junk, "%d", k);

a.add(junk);


return ( a );

}


////////////////////////////////////////////////////////////////////////


ConcatString & operator<<(ConcatString & a, long long k)

{

char junk[128];

sprintf(junk, "%lld", k);

a.add(junk);


return ( a );

}


////////////////////////////////////////////////////////////////////////


ConcatString & operator<<(ConcatString & a, double x)

{

char junk[64];

sprintf(junk, a.float_format(), x);

a.add(junk);


return ( a );

}


////////////////////////////////////////////////////////////////////////


ostream & operator<<(ostream & out, const ConcatString & s)

{

out << (s.text());

return ( out );

}


////////////////////////////////////////////////////////////////////////





