// *=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*
// ** Copyright UCAR (c) 1992 - 2007
// ** University Corporation for Atmospheric Research (UCAR)
// ** National Center for Atmospheric Research (NCAR)
// ** Research Applications Lab (RAL)
// ** P.O.Box 3000, Boulder, Colorado, 80307-3000, USA
// *=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*

///////////////////////////////////////////////////////////////////////////////

#ifndef  __VX_WRFMODE_ENGINE_H__
#define  __VX_WRFMODE_ENGINE_H__

///////////////////////////////////////////////////////////////////////////////

#include "vx_wrfmode/interest.h"
#include "vx_wrfmode/set.h"
#include "vx_wrfmode/WrfMode_Conf.h"

#include "vx_wrfdata/vx_wrfdata.h"
#include "vx_math/vx_math.h"
#include "vx_color/vx_color.h"
#include "vx_data_grids/grid.h"
#include "vx_met_util/vx_met_util.h"

///////////////////////////////////////////////////////////////////////////////
//
// Constants used to format the output text files
//
///////////////////////////////////////////////////////////////////////////////

static const int integer_precision     = 0;

static const int cts_width             = 10;

static const int default_model_width   = 6;

static const int fcst_lead_width       = 9;  // Integer
static const int fcst_valid_width      = 15; // Integer
static const int fcst_accum_width      = 10; // Integer

static const int lead2_width           = 8;  // Integer
static const int obs_valid_width       = 15; // Integer
static const int obs_accum_width       = 9;  // Integer

static const int rad_width             = 8;
static const int thresh_width          = 11;
static const int object_id_width       = 11; // Formatted
static const int object_cat_width      = 11; // Formatted

static const int centroid_xy_width     = 12;
static const int centroid_latlon_width = 12;
static const int axis_angle_width      = 12;
static const int length_width          = 12;
static const int width_width           = 12;
static const int area_width            = 12; // Integer
static const int area_filter_width     = 12; // Integer
static const int curvature_width       = 12;
static const int curvature_xy_width    = 12;
static const int complexity_width      = 12;
static const int intensity_width       = 14;

static const int centroid_dist_width              = 14;
static const int boundary_dist_width              = 14;
static const int convex_hull_dist_width           = 16;
static const int angle_diff_width                 = 12;
static const int area_ratio_width                 = 12;
static const int intersection_area_width          = 18; // Integer
static const int union_area_width                 = 12; // Integer
static const int symmetric_diff_width             = 14; // Integer
static const int intersection_over_area_width     = 22;
static const int complexity_ratio_width           = 16;
static const int percentile_intensity_ratio_width = 26;
static const int interest_width                   = 8;

static const char hms_fmt     [] = "%.2i%.2i%.2i";
static const char ymd_hms_fmt [] = "%.4i%.2i%.2i_%.2i%.2i%.2i";

///////////////////////////////////////////////////////////////////////////////

//
// R, G, B values to be used for unmatched objects in postscript output
//
static const Color unmatched_color(0, 0, 255);

//
// Standard names for raw field parameters
//
static const char grid_res_name                 [] = "grid_res";
static const char fcst_grib_code_name           [] = "fcst_grib_code";
static const char obs_grib_code_name            [] = "obs_grib_code";
static const char mask_missing_flag_name        [] = "mask_missing_flag";
static const char mask_grid_name                [] = "mask_grid";
static const char mask_grid_flag_name           [] = "mask_grid_flag";
static const char mask_poly_name                [] = "mask_poly";
static const char mask_poly_flag_name           [] = "mask_poly_flag";

//
// Standard names for object definition parameters
//
static const char fcst_raw_threshold_name       [] = "fcst_raw_threshold";
static const char obs_raw_threshold_name        [] = "obs_raw_threshold";
static const char fcst_conv_radius_name         [] = "fcst_conv_radius";
static const char obs_conv_radius_name          [] = "obs_conv_radius";
static const char bad_data_threshold_name       [] = "bad_data_threshold";
static const char fcst_conv_threshold_name      [] = "fcst_conv_threshold";
static const char obs_conv_threshold_name       [] = "obs_conv_threshold";
static const char fcst_area_threshold_name      [] = "fcst_area_threshold";
static const char obs_area_threshold_name       [] = "obs_area_threshold";
static const char fcst_inten_perc_name          [] = "fcst_inten_perc";
static const char fcst_inten_perc_threshold_name[] = "fcst_inten_perc_threshold";
static const char obs_inten_perc_name           [] = "obs_inten_perc";
static const char obs_inten_perc_threshold_name [] = "obs_inten_perc_threshold";
static const char fcst_merge_threshold_name     [] = "fcst_merge_threshold";
static const char obs_merge_threshold_name      [] = "obs_merge_threshold";

//
// Standard names for matching/merging flags
//
static const char fcst_merge_flag_name          [] = "fcst_merge_flag";
static const char obs_merge_flag_name           [] = "obs_merge_flag";
static const char match_flag_name               [] = "match_flag";
static const char max_centroid_dist_name        [] = "max_centroid_dist";

//
// Standard names for fuzzy engine weights
//
static const char centroid_dist_weight_name     [] = "centroid_dist_weight";
static const char boundary_dist_weight_name     [] = "boundary_dist_weight";
static const char convex_hull_dist_weight_name  [] = "convex_hull_dist_weight";
static const char angle_diff_weight_name        [] = "angle_diff_weight";
static const char area_ratio_weight_name        [] = "area_ratio_weight";
static const char int_area_ratio_weight_name    [] = "int_area_ratio_weight";
static const char complexity_ratio_weight_name  [] = "complexity_ratio_weight";
static const char intensity_percentile_name     [] = "intensity_percentile";
static const char intensity_ratio_weight_name   [] = "intensity_ratio_weight";

//
// Standard names for interest functions
//
static const char centroid_dist_if_name         [] = "centroid_dist_if";
static const char boundary_dist_if_name         [] = "boundary_dist_if";
static const char convex_hull_dist_if_name      [] = "convex_hull_dist_if";
static const char angle_diff_if_name            [] = "angle_diff_if";
static const char ratio_if_name                 [] = "ratio_if";
static const char area_ratio_if_name            [] = "area_ratio_if";
static const char int_area_ratio_if_name        [] = "int_area_ratio_if";
static const char complexity_ratio_if_name      [] = "complexity_ratio_if";
static const char intensity_ratio_if_name       [] = "intensity_ratio_if";

//
// Standard names for confidence functions
//
static const char aspect_ratio_conf_name        [] = "aspect_ratio_conf";
static const char area_ratio_conf_name          [] = "area_ratio_conf";

//
// Standard names for interest thresholds
//
static const char total_interest_threshold_name [] = "total_interest_threshold";
static const char print_interest_threshold_name [] = "print_interest_threshold";

//
// Standard names for misc items
//
static const char zero_border_size_name         [] = "zero_border_size";
static const char raw_color_table_name          [] = "raw_color_table";
static const char mode_color_table_name         [] = "mode_color_table";
static const char ncep_defaults_name            [] = "ncep_defaults";

//
// Formatting variables for writing engine output
//
static const int stat_missing_value   = -9999;

static const char sep[]               = " ";

///////////////////////////////////////////////////////////////////////////////

static const int max_singles = 500;

///////////////////////////////////////////////////////////////////////////////

struct DistributionPercentiles  {
   double p10;
   double p25;
   double p50;
   double p75;
   double p90;
   double pth;
   double sum;
};

struct InterestInfo {
   int    fcst_number;
   int    obs_number;
   int    pair_number;
   double interest_value;
};

///////////////////////////////////////////////////////////////////////////////

typedef double (*ConfidenceFunc) (double);

///////////////////////////////////////////////////////////////////////////////

class Engine {

   private:

      Engine(const Engine &);
      Engine & operator=(const Engine &);

      void init_from_scratch();

   public:

      Engine();
     ~Engine();

      //
      // Non-const functions
      //

      void clear_features();
      void clear_colors();

      void set(const char *fcst_filename, const char * obs_filename);
      void set(const WrfData &fcst_wd, const WrfData &obs_wd);
      void set_fcst(const char *fcst_filename);
      void set_fcst(const WrfData &fcst_wd);
      void set_obs(const char *obs_filename);
      void set_obs(const WrfData &obs_wd);

      int two_to_one(int, int) const;

      void do_fcst_filter();
      void do_obs_filter();

      void do_fcst_convolution();
      void do_obs_convolution();

      void do_fcst_thresholding();
      void do_obs_thresholding();

      void do_fcst_splitting();
      void do_obs_splitting();

      void do_fcst_merging();
      void do_obs_merging();

      void do_fcst_merging(const char *);
      void do_obs_merging(const char *);

      void do_matching();

      //
      // Perform no matching, but still define the single features
      // for each simple object
      //

      void do_no_match();      
      
      //
      // Perform merging and matching in one step based on the
      // interest threshold
      //

      void do_match_merge();

      //
      // Perform merging of the forecast and observation fields
      // based on a lower convolution threshold
      //

      void do_fcst_merge_thresh();
      void do_obs_merge_thresh();

      //
      // Perform merging of the forecast and observation fields
      // using a Fuzzy Engine
      //

      void do_fcst_merge_engine(const char *);
      void do_obs_merge_engine(const char *);

      //
      // Perform match between the forecast and observation field
      // allowing no merging in the observation field
      //

      void do_match_fcst_merge();

      //
      // Perform match between the forecast and observation field
      // allowing no merging in either field
      //

      void do_match_only();

      void do_fcst_comp_splitting();
      void do_obs_comp_splitting();

      //
      // Object holding default configuration values
      //

      WrfMode_Conf wconf;

      void process_engine_config();

      //
      // SingleThresh variables to store the thesholds
      // specified in the config file
      //
      SingleThresh fcst_raw_threshold;
      SingleThresh obs_raw_threshold;
      
      SingleThresh fcst_conv_threshold;
      SingleThresh obs_conv_threshold;
      
      SingleThresh fcst_area_threshold;
      SingleThresh obs_area_threshold;
      
      SingleThresh fcst_inten_perc_threshold;
      SingleThresh obs_inten_perc_threshold;
      
      SingleThresh fcst_merge_threshold;
      SingleThresh obs_merge_threshold;

      //
      // Data
      //
      
      char fcst_field_name[max_str_len];
      char obs_field_name[max_str_len];
      char fcst_level_name[max_str_len];
      char obs_level_name[max_str_len];

      Grid * grid_mask;
      Polyline * poly_mask;

      int need_fcst_filter;
      int need_obs_filter;

      int need_fcst_conv;
      int need_obs_conv;

      int need_fcst_thresh;
      int need_obs_thresh;

      int need_fcst_split;
      int need_obs_split;

      int need_fcst_merge;
      int need_obs_merge;

      int need_match;

      int need_fcst_comp_split;
      int need_obs_comp_split;

      WrfData * fcst_raw;
      WrfData * fcst_filter;
      WrfData * fcst_conv;
      WrfData * fcst_mask;
      WrfData * fcst_split;
      WrfData * fcst_comp_split;

      WrfData * obs_raw;
      WrfData * obs_filter;
      WrfData * obs_conv;
      WrfData * obs_mask;
      WrfData * obs_split;
      WrfData * obs_comp_split;

      Engine * fcst_engine;
      Engine * obs_engine;

      int n_fcst;
      int n_obs;

      SingleFeature * fcst_single;
      SingleFeature * obs_single;

      PairFeature * pair;

      Color fcst_color [max_singles];
      Color obs_color [max_singles];
      ColorTable ctable;

      InterestInfo info[max_singles*max_singles];

      int get_info_index(int) const;

      int get_unmatched_fcst() const;

      int get_unmatched_obs() const;

      SetCollection collection;
};

///////////////////////////////////////////////////////////////////////////////

extern double total_interest(WrfMode_Conf &, const PairFeature &);
extern double total_interest_print(WrfMode_Conf &, const PairFeature &, ostream *);

extern void write_engine_stats(Engine &, const Grid &, ostream &);
extern void write_header(Engine &, ostream &);
extern void write_header_columns(Engine &, ostream &);
extern void write_fcst_single(Engine &, const int, const Grid &, ostream &);
extern void write_obs_single(Engine &, const int, const Grid &, ostream &);
extern void write_pair(Engine &, const int, const int, ostream &);
extern void write_fcst_composite(Engine &, const int, const Grid &, ostream &);
extern void write_obs_composite(Engine &, const int, const Grid &, ostream &);
extern void write_composite_pair(Engine &, const int, const int, ostream &);

extern void calc_fcst_comp_ch_mask(const Engine &, WrfData &);
extern void calc_obs_comp_ch_mask(const Engine &, WrfData &);

extern void calc_fcst_composite_mask(const Engine &, WrfData &, const int);
extern void calc_obs_composite_mask(const Engine &, WrfData &, const int);

extern void parse_threshold_info(const char *, SingleThresh &);

///////////////////////////////////////////////////////////////////////////////

#endif   // __VX_WRFMODE_ENGINE_H__

///////////////////////////////////////////////////////////////////////////////
